@@ -25,18 +25,14 @@
 #include <config.h>
 #endif
 
-#include <string.h>
+#define LIBSOUP_USE_UNSTABLE_REQUEST_API
 
 #include "soup-request-data.h"
-#include "soup.h"
-#include "soup-misc-private.h"
 
-/**
- * SECTION:soup-request-data
- * @short_description: SoupRequest support for "data" URIs
- *
- * #SoupRequestData implements #SoupRequest for "data" URIs.
- */
+#include "soup-requester.h"
+#include "soup-misc-private.h"
+#include <libsoup/soup.h>
+#include <glib/gi18n.h>
 
 G_DEFINE_TYPE (SoupRequestData, soup_request_data, SOUP_TYPE_REQUEST)
 
@@ -96,7 +92,7 @@ soup_request_data_send (SoupRequest   *request,
 			end = comma;
 
 		if (end != start)
-			data->priv->content_type = soup_uri_decoded_copy (start, end - start, NULL);
+			data->priv->content_type = uri_decoded_copy (start, end - start);
 	}
 
 	memstream = g_memory_input_stream_new ();
@@ -105,14 +101,12 @@ soup_request_data_send (SoupRequest   *request,
 		start = comma + 1;
 
 	if (*start) {
-		int decoded_length = 0;
-		guchar *buf = (guchar *) soup_uri_decoded_copy (start, strlen (start),
-								&decoded_length);
+		guchar *buf = (guchar *) soup_uri_decode (start);
 
 		if (base64)
 			buf = g_base64_decode_inplace ((gchar*) buf, &data->priv->content_length);
 		else
-			data->priv->content_length = decoded_length;
+			data->priv->content_length = strlen ((const char *) buf);
 
 		g_memory_input_stream_add_data (G_MEMORY_INPUT_STREAM (memstream),
 						buf, data->priv->content_length,
