@@ -7,7 +7,6 @@
 #define SOUP_SESSION_H 1
 
 #include <libsoup/soup-types.h>
-#include <libsoup/soup-address.h>
 #include <libsoup/soup-message.h>
 
 G_BEGIN_DECLS
@@ -60,9 +59,7 @@ typedef struct {
 
 GType soup_session_get_type (void);
 
-#define SOUP_SESSION_LOCAL_ADDRESS          "local-address"
 #define SOUP_SESSION_PROXY_URI              "proxy-uri"
-#define SOUP_SESSION_PROXY_RESOLVER         "proxy-resolver"
 #define SOUP_SESSION_MAX_CONNS              "max-conns"
 #define SOUP_SESSION_MAX_CONNS_PER_HOST     "max-conns-per-host"
 #define SOUP_SESSION_USE_NTLM               "use-ntlm"
@@ -70,7 +67,6 @@ GType soup_session_get_type (void);
 #define SOUP_SESSION_SSL_USE_SYSTEM_CA_FILE "ssl-use-system-ca-file"
 #define SOUP_SESSION_TLS_DATABASE           "tls-database"
 #define SOUP_SESSION_SSL_STRICT             "ssl-strict"
-#define SOUP_SESSION_TLS_INTERACTION        "tls-interaction"
 #define SOUP_SESSION_ASYNC_CONTEXT          "async-context"
 #define SOUP_SESSION_USE_THREAD_CONTEXT     "use-thread-context"
 #define SOUP_SESSION_TIMEOUT                "timeout"
@@ -84,12 +80,7 @@ GType soup_session_get_type (void);
 #define SOUP_SESSION_HTTP_ALIASES       "http-aliases"
 #define SOUP_SESSION_HTTPS_ALIASES      "https-aliases"
 
-SOUP_AVAILABLE_IN_2_42
-SoupSession    *soup_session_new              (void);
-
-SOUP_AVAILABLE_IN_2_42
-SoupSession    *soup_session_new_with_options (const char *optname1,
-					       ...) G_GNUC_NULL_TERMINATED;
+GMainContext   *soup_session_get_async_context(SoupSession           *session);
 
 void            soup_session_queue_message    (SoupSession           *session,
 					       SoupMessage           *msg,
@@ -111,101 +102,30 @@ void            soup_session_cancel_message   (SoupSession           *session,
 					       guint                  status_code);
 void            soup_session_abort            (SoupSession           *session);
 
-GMainContext   *soup_session_get_async_context(SoupSession           *session);
-
-SOUP_AVAILABLE_IN_2_42
-void            soup_session_send_async       (SoupSession           *session,
-					       SoupMessage           *msg,
-					       GCancellable          *cancellable,
-					       GAsyncReadyCallback    callback,
-					       gpointer               user_data);
-SOUP_AVAILABLE_IN_2_42
-GInputStream   *soup_session_send_finish      (SoupSession           *session,
-					       GAsyncResult          *result,
-					       GError               **error);
-SOUP_AVAILABLE_IN_2_42
-GInputStream   *soup_session_send             (SoupSession           *session,
-					       SoupMessage           *msg,
-					       GCancellable          *cancellable,
-					       GError               **error);
-
-#ifndef SOUP_DISABLE_DEPRECATED
-/* SOUP_AVAILABLE_IN_2_30 -- this trips up gtkdoc-scan */
-SOUP_DEPRECATED_IN_2_38_FOR (soup_session_prefetch_dns)
 void            soup_session_prepare_for_uri  (SoupSession           *session,
 					       SoupURI               *uri);
-#endif
 
-SOUP_AVAILABLE_IN_2_38
-void            soup_session_prefetch_dns     (SoupSession           *session,
-					       const char            *hostname,
-					       GCancellable          *cancellable,
-					       SoupAddressCallback    callback,
-					       gpointer               user_data);
-
-SOUP_AVAILABLE_IN_2_38
 gboolean        soup_session_would_redirect   (SoupSession           *session,
 					       SoupMessage           *msg);
-SOUP_AVAILABLE_IN_2_38
 gboolean        soup_session_redirect_message (SoupSession           *session,
 					       SoupMessage           *msg);
 
-SOUP_AVAILABLE_IN_2_24
 void                soup_session_add_feature            (SoupSession        *session,
 							 SoupSessionFeature *feature);
-SOUP_AVAILABLE_IN_2_24
 void                soup_session_add_feature_by_type    (SoupSession        *session,
 							 GType               feature_type);
-SOUP_AVAILABLE_IN_2_24
 void                soup_session_remove_feature         (SoupSession        *session,
 							 SoupSessionFeature *feature);
-SOUP_AVAILABLE_IN_2_24
 void                soup_session_remove_feature_by_type (SoupSession        *session,
 							 GType               feature_type);
-SOUP_AVAILABLE_IN_2_42
-gboolean            soup_session_has_feature            (SoupSession        *session,
-							 GType               feature_type);
-SOUP_AVAILABLE_IN_2_26
 GSList             *soup_session_get_features           (SoupSession        *session,
 							 GType               feature_type);
-SOUP_AVAILABLE_IN_2_26
 SoupSessionFeature *soup_session_get_feature            (SoupSession        *session,
 							 GType               feature_type);
-SOUP_AVAILABLE_IN_2_28
 SoupSessionFeature *soup_session_get_feature_for_message(SoupSession        *session,
 							 GType               feature_type,
 							 SoupMessage        *msg);
 
-SOUP_AVAILABLE_IN_2_42
-SoupRequest     *soup_session_request          (SoupSession  *session,
-						const char   *uri_string,
-						GError      **error);
-SOUP_AVAILABLE_IN_2_42
-SoupRequest     *soup_session_request_uri      (SoupSession  *session,
-						SoupURI      *uri,
-						GError      **error);
-SOUP_AVAILABLE_IN_2_42
-SoupRequestHTTP *soup_session_request_http     (SoupSession  *session,
-						const char   *method,
-						const char   *uri_string,
-						GError      **error);
-SOUP_AVAILABLE_IN_2_42
-SoupRequestHTTP *soup_session_request_http_uri (SoupSession  *session,
-						const char   *method,
-						SoupURI      *uri,
-						GError      **error);
-
-SOUP_AVAILABLE_IN_2_42
-GQuark soup_request_error_quark (void);
-#define SOUP_REQUEST_ERROR soup_request_error_quark ()
-
-typedef enum {
-	SOUP_REQUEST_ERROR_BAD_URI,
-	SOUP_REQUEST_ERROR_UNSUPPORTED_URI_SCHEME,
-	SOUP_REQUEST_ERROR_PARSING,
-	SOUP_REQUEST_ERROR_ENCODING
-} SoupRequestError;
-
 G_END_DECLS
 
 #endif /* SOUP_SESSION_H */
