@@ -3,6 +3,24 @@
  * Copyright (C) 2007 Red Hat, Inc.
  */
 
+#include <ctype.h>
+#include <fcntl.h>
+#include <errno.h>
+#include <signal.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/stat.h>
+#include <unistd.h>
+
+#include <glib.h>
+#include <libsoup/soup-address.h>
+#include <libsoup/soup-message.h>
+#include <libsoup/soup-misc.h>
+#include <libsoup/soup-server.h>
+#include <libsoup/soup-session-async.h>
+#include <libsoup/soup-session-sync.h>
+
 #include "test-utils.h"
 
 static char *base_uri;
@@ -14,12 +32,12 @@ typedef struct {
 } SlowData;
 
 static void
-request_finished (SoupMessage *msg, gpointer data)
+request_failed (SoupMessage *msg, gpointer data)
 {
 	SlowData *sd = data;
 
-	g_source_destroy (sd->timeout);
-	g_source_unref (sd->timeout);
+	if (SOUP_STATUS_IS_TRANSPORT_ERROR (msg->status_code))
+		g_source_destroy (sd->timeout);
 	g_free (sd);
 }
 
@@ -65,11 +83,10 @@ server_callback (SoupServer *server, SoupMessage *msg,
 	sd->server = server;
 	sd->msg = msg;
 	sd->timeout = soup_add_timeout (
-		g_main_context_get_thread_default (),
+		soup_server_get_async_context (server),
 		200, add_body_chunk, sd);
-	g_source_ref (sd->timeout);
 	g_signal_connect (msg, "finished",
-			  G_CALLBACK (request_finished), sd);
+			  G_CALLBACK (request_failed), sd);
 }
 
 /* Test 1: An async session in another thread with its own
@@ -85,9 +102,13 @@ static GMutex test1_mutex;
 static GMainLoop *test1_loop;
 
 static void
-do_test1 (gconstpointer data)
+do_test1 (int n, gboolean use_thread_context)
 {
-	gboolean use_thread_context = GPOINTER_TO_INT (data);
+	debug_printf (1, "\nTest %d: blocking the main thread does not block other thread\n", n);
+	if (use_thread_context)
+		debug_printf (1, "(Using g_main_context_push_thread_default())\n");
+	else
+		debug_printf (1, "(Using SOUP_SESSION_ASYNC_CONTEXT)\n");
 
 	test1_loop = g_main_loop_new (NULL, FALSE);
 	g_idle_add (idle_start_test1_thread, GINT_TO_POINTER (use_thread_context));
@@ -108,8 +129,9 @@ idle_start_test1_thread (gpointer use_thread_context)
 	if (g_cond_wait_until (&test1_cond, &test1_mutex, time))
 		g_thread_join (thread);
 	else {
-		soup_test_assert (FALSE, "timeout");
+		debug_printf (1, "  timeout!\n");
 		g_thread_unref (thread);
+		errors++;
 	}
 
 	g_mutex_unlock (&test1_mutex);
@@ -154,7 +176,11 @@ test1_thread (gpointer use_thread_context)
 	debug_printf (1, "  send_message\n");
 	msg = soup_message_new ("GET", uri);
 	soup_session_send_message (session, msg);
-	soup_test_assert_message_status (msg, SOUP_STATUS_OK);
+	if (msg->status_code != SOUP_STATUS_OK) {
+		debug_printf (1, "    unexpected status: %d %s\n",
+			      msg->status_code, msg->reason_phrase);
+		errors++;
+	}
 	g_object_unref (msg);
 
 	debug_printf (1, "  queue_message\n");
@@ -164,7 +190,11 @@ test1_thread (gpointer use_thread_context)
 	soup_session_queue_message (session, msg, test1_finished, loop);
 	g_main_loop_run (loop);
 	g_main_loop_unref (loop);
-	soup_test_assert_message_status (msg, SOUP_STATUS_OK);
+	if (msg->status_code != SOUP_STATUS_OK) {
+		debug_printf (1, "    unexpected status: %d %s\n",
+			      msg->status_code, msg->reason_phrase);
+		errors++;
+	}
 	g_object_unref (msg);
 
 	soup_test_session_abort_unref (session);
@@ -184,15 +214,20 @@ test1_thread (gpointer use_thread_context)
 static gboolean idle_test2_fail (gpointer user_data);
 
 static void
-do_test2 (gconstpointer data)
+do_test2 (int n, gboolean use_thread_context)
 {
-	gboolean use_thread_context = GPOINTER_TO_INT (data);
 	guint idle;
 	GMainContext *async_context;
 	SoupSession *session;
 	char *uri;
 	SoupMessage *msg;
 
+	debug_printf (1, "\nTest %d: a session with its own context is independent of the main loop.\n", n);
+	if (use_thread_context)
+		debug_printf (1, "(Using g_main_context_push_thread_default())\n");
+	else
+		debug_printf (1, "(Using SOUP_SESSION_ASYNC_CONTEXT)\n");
+
 	idle = g_idle_add_full (G_PRIORITY_HIGH, idle_test2_fail, NULL, NULL);
 
 	async_context = g_main_context_new ();
@@ -213,7 +248,11 @@ do_test2 (gconstpointer data)
 	debug_printf (1, "  send_message\n");
 	msg = soup_message_new ("GET", uri);
 	soup_session_send_message (session, msg);
-	soup_test_assert_message_status (msg, SOUP_STATUS_OK);
+	if (msg->status_code != SOUP_STATUS_OK) {
+		debug_printf (1, "    unexpected status: %d %s\n",
+			      msg->status_code, msg->reason_phrase);
+		errors++;
+	}
 	g_object_unref (msg);
 
 	soup_test_session_abort_unref (session);
@@ -228,7 +267,8 @@ do_test2 (gconstpointer data)
 static gboolean
 idle_test2_fail (gpointer user_data)
 {
-	soup_test_assert (FALSE, "idle ran");
+	debug_printf (1, "  idle ran!\n");
+	errors++;
 	return FALSE;
 }
 
@@ -257,13 +297,15 @@ multi_msg_finished (SoupSession *session, SoupMessage *msg, gpointer user_data)
 }
 
 static void
-do_multicontext_test (void)
+do_multicontext_test (int n)
 {
 	SoupSession *session;
 	SoupMessage *msg1, *msg2;
 	GMainContext *context1, *context2;
 	GMainLoop *loop1, *loop2;
 
+	debug_printf (1, "\nTest %d: Using multiple async contexts\n", n);
+
 	session = soup_test_session_new (SOUP_TYPE_SESSION_ASYNC,
 					 SOUP_SESSION_USE_THREAD_CONTEXT, TRUE,
 					 NULL);
@@ -294,26 +336,36 @@ do_multicontext_test (void)
 	g_main_loop_run (loop1);
 	g_main_context_pop_thread_default (context1);
 
-	if (!g_object_get_data (G_OBJECT (msg1), "started"))
-		soup_test_assert (FALSE, "msg1 not started");
-	if (g_object_get_data (G_OBJECT (msg2), "started"))
-		soup_test_assert (FALSE, "msg2 started while loop1 was running");
+	if (!g_object_get_data (G_OBJECT (msg1), "started")) {
+		debug_printf (1, "  msg1 not started??\n");
+		errors++;
+	}
+	if (g_object_get_data (G_OBJECT (msg2), "started")) {
+		debug_printf (1, "  msg2 started while loop1 was running!\n");
+		errors++;
+	}
 
 	g_main_context_push_thread_default (context2);
 	g_main_loop_run (loop2);
 	g_main_context_pop_thread_default (context2);
 
-	if (g_object_get_data (G_OBJECT (msg1), "finished"))
-		soup_test_assert (FALSE, "msg1 finished while loop2 was running");
-	if (!g_object_get_data (G_OBJECT (msg2), "finished"))
-		soup_test_assert (FALSE, "msg2 not finished");
+	if (g_object_get_data (G_OBJECT (msg1), "finished")) {
+		debug_printf (1, "  msg1 finished while loop2 was running!\n");
+		errors++;
+	}
+	if (!g_object_get_data (G_OBJECT (msg2), "finished")) {
+		debug_printf (1, "  msg2 not finished??\n");
+		errors++;
+	}
 
 	g_main_context_push_thread_default (context1);
 	g_main_loop_run (loop1);
 	g_main_context_pop_thread_default (context1);
 
-	if (!g_object_get_data (G_OBJECT (msg1), "finished"))
-		soup_test_assert (FALSE, "msg1 not finished");
+	if (!g_object_get_data (G_OBJECT (msg1), "finished")) {
+		debug_printf (1, "  msg1 not finished??\n");
+		errors++;
+	}
 
 	g_object_unref (msg1);
 	g_object_unref (msg2);
@@ -330,28 +382,23 @@ int
 main (int argc, char **argv)
 {
 	SoupServer *server;
-	SoupURI *uri;
-	int ret;
 
 	test_init (argc, argv, NULL);
 
-	server = soup_test_server_new (SOUP_TEST_SERVER_IN_THREAD);
+	server = soup_test_server_new (TRUE);
 	soup_server_add_handler (server, NULL, server_callback, NULL, NULL);
-	uri = soup_test_server_get_uri (server, "http", NULL);
-	base_uri = soup_uri_to_string (uri, FALSE);
-	soup_uri_free (uri);
+	base_uri = g_strdup_printf ("http://127.0.0.1:%u/",
+				    soup_server_get_port (server));
 
-	g_test_add_data_func ("/context/blocking/explicit", GINT_TO_POINTER (FALSE), do_test1);
-	g_test_add_data_func ("/context/blocking/thread-default", GINT_TO_POINTER (TRUE), do_test1);
-	g_test_add_data_func ("/context/nested/explicit", GINT_TO_POINTER (FALSE), do_test2);
-	g_test_add_data_func ("/context/nested/thread-default", GINT_TO_POINTER (TRUE), do_test2);
-	g_test_add_func ("/context/multiple", do_multicontext_test);
-
-	ret = g_test_run ();
+	do_test1 (1, FALSE);
+	do_test1 (2, TRUE);
+	do_test2 (3, FALSE);
+	do_test2 (4, TRUE);
+	do_multicontext_test (5);
 
 	g_free (base_uri);
 	soup_test_server_quit_unref (server);
 
 	test_cleanup ();
-	return ret;
+	return errors != 0;
 }
